/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.gui.calcheadviews;

import isac.util.formulae.CalcHead;
import isac.util.formulae.KEStoreID;
import isac.util.formulae.ModelItem;
import isac.util.formulae.ModelItemList;
import isac.util.formulae.CalcHeadSimpleID;
import isac.util.formulae.Specification;

import java.awt.Color;
import java.util.*;

import javax.swing.*;
import javax.swing.text.*;

/**
 * Abstract class that specifies the commons of all different views of a
 * <code>CalcHead</code>.
 * 
 * @author Mario Hochreiter
 * @version 1.0
 * 
 * WN0504 there will be only _ONE_ CalcHead view forever...
 * @see isac.util.formulae.CalcHead#VIEWSTYLE_FULL because
 *      isac.util.formulae.CalcHead#VIEWSTYLE_SINGLELINE is used _NOT_ to show
 *      the CalcHeadPanel
 */
public abstract class CalcHeadView extends JPanel implements CHeadView {
    protected JLabel label_given_;

    protected JLabel label_where_;

    protected JLabel label_find_;

    protected JLabel label_theory_;

    protected JLabel label_method_;

    protected JLabel label_problem_;

    protected JLabel label_relate_;

    protected JTextPane txt_given_;

    protected JTextPane txt_where_;

    protected JTextPane txt_find_;

    protected JTextPane txt_theory_;

    protected JTextPane txt_method_;

    protected JTextPane txt_problem_;

    protected JTextPane txt_relate_;

    protected Locale locale_;

    protected ResourceBundle res_bundle_;

    public CalcHeadView(Locale locale) {
        this.locale_ = locale;
        res_bundle_ = ResourceBundle.getBundle("CalcHeadPanel", locale);
    }

    /**
     * @param correct
     *            If true then colorize the label green otherwise red.
     * @param labelGiven
     *            Label that should be colored
     */
    protected void colorizeLabel(boolean correct, JLabel label) {
        if (correct)
            label.setForeground(new Color(40, 210, 40));
        else
            label.setForeground(Color.RED);
    }

    /**
     * @return False if a status was not CORRECT
     * 
     * WN040908 part of undesired behaviour: the status (below 'attributes') of
     * items should be displayed _per item_ and not for some collection (eg. for
     * 'given')
     */
    protected boolean checkAndFormatCalcHeadItems(Iterator it, JTextPane txt) {
        boolean correct = true;
        StyledDocument doc = txt.getStyledDocument();
        while (it.hasNext()) {
            Object o = it.next();
            try {
                if (o instanceof ModelItem) {
                    ModelItem chi = (ModelItem) o;

                    SimpleAttributeSet attributes = new SimpleAttributeSet();
                    if (chi.getStatus() != CalcHead.MODEL_ITEM_CORRECT) {
                        StyleConstants.setForeground(attributes, Color.red);
                        correct = false;
                    }
                    doc.insertString(doc.getLength(), chi.toSMLString(),
                            attributes);
                } else if (o instanceof String) {
                    String s = (String) o;
                    doc.insertString(doc.getLength(), s, null);
                }
                if (it.hasNext())
                    doc.insertString(doc.getLength(), ",", null);
            } catch (BadLocationException e) {
                e.printStackTrace();
            }
        }
        return correct;
    }

    /** Clear all strings in the text fields */
    protected void clearTextFields() {
        if (txt_given_ != null)
            txt_given_.setText("");
        if (txt_where_ != null)
            txt_where_.setText("");
        if (txt_find_ != null)
            txt_find_.setText("");
        if (txt_relate_ != null)
            txt_relate_.setText("");
        if (txt_theory_ != null)
            txt_theory_.setText("");
        if (txt_problem_ != null)
            txt_problem_.setText("");
        if (txt_method_ != null)
            txt_method_.setText("");
        //WN040908 missing: belongs-to
    }

    /**
     * parse the Text Field txtGiven which can have multiple mathTexts seperated
     * by a seperator char(a comma at the moment) so that as a result those
     * multiple mathTexts are in a String[] where every element is one mathText
     * 
     * @param seperator:
     *            the char that seperates multiple mathTexts
     * @return String[] which contains all mathTexts of the text Field
     * @author Alois Kirchsteiger 10.09.2004 10:49:30
     */
    protected String[] parseTxtGivenToStringArray(char seperator) {
        String given = txt_given_.getText();
        String seperatorString = String.valueOf(seperator);
        return given.split(seperatorString);
    }

    /**
     * this method inserts the mathTexts from the TextField txtGiven in the
     * calcHead
     * 
     * @param calcHead
     *            is modified in the method!
     * @author Alois Kirchsteiger 10.09.2004 ~12
     */
    protected void insertTxtGivenInCalcHead(CalcHead calcHead) {
        String[] given = parseTxtGivenToStringArray(',');
        ModelItemList givenList = new ModelItemList();
        for (int i = 0; i < given.length; i++) {
            ModelItem givenItem = new ModelItem();
            givenItem.setText(given[i]);
            givenList.addItem(givenItem);
        }
        calcHead.setGiven(givenList);
    }

    /**
     * parse the Text Field txtWhere which can have multiple mathTexts seperated
     * by a seperator char(a comma at te moment) so that as a result those
     * multiple mathTexts are in a String[] where every element is one mathText
     * 
     * @param seperator:
     *            the char that seperates multiple mathTexts
     * @return String[] which contains all mathTexts of the text Field
     * @author Alois Kirchsteiger 10.09.2004 ~11
     */
    protected String[] parseTxtWhereToStringArray(char seperator) {
        String where = txt_where_.getText();
        String seperatorString = String.valueOf(seperator);
        return where.split(seperatorString);
    }

    /**
     * this method inserts the mathTexts from the TextField txtWhere in the
     * calcHead
     * 
     * @param calcHead
     *            is modified in the method! FIXXXME.WN050624 replace param
     *            'CalcHead' by param 'Model'
     * @author Alois Kirchsteiger 10.09.2004 ~12
     *  
     */
    protected void insertTxtWhereInCalcHead(CalcHead calcHead) {
        String[] where = parseTxtWhereToStringArray(',');
        ModelItemList whereList = new ModelItemList();
        for (int i = 0; i < where.length; i++) {
            ModelItem whereItem = new ModelItem();
            whereItem.setText(where[i]);
            whereList.addItem(whereItem);
        }
        calcHead.setGiven(whereList);
    }

    /**
     * parse the Text Field txtFind which can have multiple mathTexts seperated
     * by a seperator char(a comma at te moment) so that as a result those
     * multiple mathTexts are in a String[] where every element is one mathText
     * 
     * @param seperator:
     *            the char that seperates multiple mathTexts
     * @return String[] which contains all mathTexts of the text Field
     * @author Alois Kirchsteiger 10.09.2004 ~11
     */
    protected String[] parseTxtFindToStringArray(char seperator) {
        String find = txt_find_.getText();
        String seperatorString = String.valueOf(seperator);
        return find.split(seperatorString);
    }

    /**
     * this method inserts the mathTexts from the TextField txtFind in the
     * calcHead
     * 
     * @param calcHead
     *            is modified in the method!
     * @author Alois Kirchsteiger 10.09.2004 ~12
     */
    protected void insertTxtFindInCalcHead(CalcHead calcHead) {
        String[] find = parseTxtFindToStringArray(',');
        ModelItemList findList = new ModelItemList();
        for (int i = 0; i < find.length; i++) {
            ModelItem findItem = new ModelItem();
            findItem.setText(find[i]);
            findList.addItem(findItem);
        }
        calcHead.setGiven(findList);
    }

    /**
     * parse the Text Field txtRelate which can have multiple mathTexts
     * seperated by a seperator char(a comma at te moment) so that as a result
     * those multiple mathTexts are in a String[] where every element is one
     * mathText
     * 
     * @param seperator:
     *            the char that seperates multiple mathTexts
     * @return String[] which contains all mathTexts of the text Field
     * @author Alois Kirchsteiger 10.09.2004 ~11
     */
    protected String[] parseTxtRelateToStringArray(char seperator) {
        String relate = txt_relate_.getText();
        String seperatorString = String.valueOf(seperator);
        return relate.split(seperatorString);
    }

    /**
     * this method inserts the mathTexts from the TextField txtRelate in the
     * calcHead
     * 
     * @param calcHead
     *            is modified in the method!
     * @author Alois Kirchsteiger 10.09.2004 ~12
     */
    protected void insertTxtRelateInCalcHead(CalcHead calcHead) {
        String[] relate = parseTxtRelateToStringArray(',');
        ModelItemList relateList = new ModelItemList();
        for (int i = 0; i < relate.length; i++) {
            ModelItem relateItem = new ModelItem();
            relateItem.setText(relate[i]);
            relateList.addItem(relateItem);
        }
        calcHead.setGiven(relateList);
    }

    /**
     * parse the Text Field txtProblem which can have multiple mathTexts
     * seperated by a seperator char(a comma at te moment) so that as a result
     * those multiple mathTexts are in a String[] where every element is one
     * mathText
     * 
     * @param seperator:
     *            the char that seperates multiple mathTexts
     * @return String[] which contains all mathTexts of the text Field
     * @author Alois Kirchsteiger 10.09.2004 ~11
     */
    protected String[] parseTxtProblemToStringArray(char seperator) {
        String problem = txt_problem_.getText();
        String seperatorString = String.valueOf(seperator);
        return problem.split(seperatorString);
    }

    /**
     * insert the mathTexts from the TextField txtProblem in the CalcHead
     * 
     * @param calcHead
     */
    protected void insertTxtProblemInCalcHeadSpecification(CalcHead calcHead) {
        Specification specification = calcHead.getSpecification();
        String[] problem = parseTxtProblemToStringArray(',');
        KEStoreID problemCalcCompoundID = new KEStoreID();
        for (int i = 0; i < problem.length; i++) {
            problemCalcCompoundID.addString(problem[i]);
        }
        specification.setProblem(problemCalcCompoundID);
        calcHead.setSpecification(specification);
    }

    /**
     * parse the Text Field txtMethod which can have multiple mathTexts
     * seperated by a seperator char(a comma at te moment) so that as a result
     * those multiple mathTexts are in a String[] where every element is one
     * mathText
     * 
     * @param seperator:
     *            the char that seperates multiple mathTexts
     * @return String[] which contains all mathTexts of the text Field
     * @author Alois Kirchsteiger 10.09.2004 ~11
     */
    protected String[] parseTxtMethodToStringArray(char seperator) {
        String method = txt_method_.getText();
        String seperatorString = String.valueOf(seperator);
        return method.split(seperatorString);
    }

    /**
     * insert the mathTexts from the TextField txtMethod in the CalcHead
     * 
     * @param calcHead
     */
    protected void insertTxtMethodInCalcHeadSpecification(CalcHead calcHead) {
        Specification specification = calcHead.getSpecification();
        String[] method = parseTxtMethodToStringArray(',');
        KEStoreID methodCalcCompoundID = new KEStoreID();
        for (int i = 0; i < method.length; i++) {
            methodCalcCompoundID.addString(method[i]);
        }
        specification.setProblem(methodCalcCompoundID);
        calcHead.setSpecification(specification);
    }

    /**
     * parse the Text Field txtTheory which can have multiple mathTexts
     * seperated by a seperator char(a comma at te moment) so that as a result
     * those multiple mathTexts are in a String[] where every element is one
     * mathText
     * 
     * @param seperator:
     *            the char that seperates multiple mathTexts
     * @return String[] which contains all mathTexts of the text Field
     * @author Alois Kirchsteiger 10.09.2004 ~11
     */
    protected String[] parseTxtTheoryToStringArray(char seperator) {
        String theory = txt_theory_.getText();
        String seperatorString = String.valueOf(seperator);
        return theory.split(seperatorString);
    }

    /**
     * insert the mathTexts from the TextField txtTheory in the CalcHead
     * 
     * @param calcHead
     */
    protected void insertTxtTheoryInCalcHeadSpecification(CalcHead calcHead) {
        Specification specification = calcHead.getSpecification();
        String theory = txt_theory_.getText();
        CalcHeadSimpleID theoryCalcHeadSimpleID = new CalcHeadSimpleID();
        theoryCalcHeadSimpleID.setID(theory);
        specification.setTheory(theoryCalcHeadSimpleID);
        calcHead.setSpecification(specification);
    }

    /**
     * inserts the whle specification (theory, method, problem) in the calcHead
     * 
     * @param calcHead
     * @author Alois Kirchsteiger 10.09.2004 13:56:25
     */

    protected void insertSpecificationFromTextFieldsInCalcHead(CalcHead calcHead) {
        insertTxtTheoryInCalcHeadSpecification(calcHead);
        insertTxtMethodInCalcHeadSpecification(calcHead);
        insertTxtProblemInCalcHeadSpecification(calcHead);
    }
}